import React from 'react';
import PropTypes from 'prop-types';
import { ActivityIndicator } from 'react-native';
import ButtonBase from '../ButtonBase/ButtonBase';
import TextLabel from '../TextLabel/TextLabel';
import Icon from '../Icon/Icon';

// Styles
import getStyles from './Button.styles';

/**
 * A generic button button control
 */
const Button = (props) => {
    const {
        contextStyles,
        disabled,
        iconName,
        indicatorSize,
        onLongPress,
        onPress,
        reverse,
        showIndicator,
        testID,
        text,
        variant,
    } = props;

    const padLeftText = !reverse && !!iconName && !!text;
    const padRightText = reverse && !!iconName && !!text;
    const padIndicator = showIndicator && (!!iconName || !!text);
    const styleArgs = {
        variant,
        padLeftText,
        padRightText,
        padIndicator,
    };
    const styles = getStyles(contextStyles, styleArgs);

    // Prepare Text node
    const textProps = {
        contextStyles: {
            text: styles.text,
        },
    };
    if (testID) { textProps.testID = `${testID}__text`; }

    // Create function but do not render as it may never be used
    const renderText = () => (
        <TextLabel { ...textProps }>
            { text }
        </TextLabel>
    );

    // Prepare Icon node
    const iconProps = {
        contextStyles: {
            icon: styles.icon,
        },
        name: iconName,
    };
    if (testID) { iconProps.testID = `${testID}__icon`; }

    // Create function but do not render as it may never be used
    const renderIcon = () => (
        <Icon { ...iconProps } />
    );

    // Prepare indicator node
    const indicatorProps = {
        color: styles.indicator.color,
        indicatorSize,
        style: styles.indicator,
    };
    if (testID) { indicatorProps.testID = `${testID}__indicator`; }

    // Create function but do not render as it may never be used
    const renderIndicator = () => (
        <ActivityIndicator { ...indicatorProps } />
    );

    // Generate button props
    const buttonProps = {
        contextStyles: {
            container: styles.container,
        },
    };
    if (disabled) {
        buttonProps.disabled = disabled;
        buttonProps.contextStyles.disabledContainer = styles.disabledContainer;
    }
    if (!disabled && onLongPress) { buttonProps.onLongPress = onLongPress; }
    if (!disabled && onPress) { buttonProps.onPress = onPress; }
    if (testID) { buttonProps.testID = `${testID}__container`; }

    return (
        <ButtonBase { ...buttonProps }>
            { (!!iconName && !reverse) && renderIcon() }
            { !!text && renderText() }
            { (!!iconName && reverse) && renderIcon() }
            { showIndicator && renderIndicator() }
        </ButtonBase>
    );
};

Button.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main button container */
        container: PropTypes.object,

        /** Styles for the container in a disabled state */
        disabledContainer: PropTypes.object,

        /** Styles for icon */
        icon: PropTypes.object,

        /** Styles for the activity indicator */
        indicator: PropTypes.object,

        /** Styles for the button text */
        text: PropTypes.object,
    }),

    /** Disable the button */
    disabled: PropTypes.bool,

    /** What icon to show */
    iconName: PropTypes.string,

    /** Size of the activity indicator */
    indicatorSize: PropTypes.oneOf([
        'small',
        'large',
    ]),

    /**
     * Called when the button is long pressed
     * @param {SyntheticEvent} event - The native long press event
     */
    onLongPress: PropTypes.func,

    /**
     * Called when the button is pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onPress: PropTypes.func,

    /** Icon location.  Defaults to left. */
    reverse: PropTypes.bool,

    /** Displays an activity indicator on the button */
    showIndicator: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Text displayed inside the button */
    text: PropTypes.string,

    /** Button variant type */
    variant: PropTypes.oneOf([
        'primary',
        'secondary',
    ]),
};

Button.defaultProps = {
    contextStyles: {},
    indicatorSize: 'small',
    variant: 'primary',
};

export default Button;
