import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { Image as NativeImage, ActivityIndicator } from 'react-native';
import Ripple from 'react-native-material-ripple';
import Icon from 'react-native-vector-icons/MaterialCommunityIcons';

// Styles
import getStyles from './Image.styles';

/**
 * A component for displaying an image.
 */
const Image = (props) => {
    const {
        contextStyles,
        disabled,
        height,
        imageSrc,
        onLongPress,
        onPress,
        placeholderIconColor,
        placeholderIconName,
        placeholderIconSize,
        showLoadingIndicator,
        stretch,
        testID,
        width,
    } = props;

    const styles = getStyles(contextStyles, height, width);
    const { loadingIndicator } = styles;

    // Create function but do not render as it may never be used
    const renderPlaceholder = () => (
        <Icon
            name={ placeholderIconName }
            size={ width || placeholderIconSize }
            color={ placeholderIconColor } />
    );

    // Create function but do not render as it may never be used
    const renderImage = () => {
        // Generate image props
        const imageProps = {
            resizeMode: stretch ? 'stretch' : 'contain',
            source: typeof imageSrc === 'string' ? { uri: imageSrc } : imageSrc,
        };
        if (testID) { imageProps.testID = `${testID}__image`; }

        // Extract image width style in case image is stretched
        const {
            width: originalWidth,
            ...restStyles
        } = styles.image;

        // Begin new image styles without image width
        const imageStyle = {
            ...restStyles,
        };

        // If we are not stretching the image
        if (!stretch) {
            // Re-apply the image width style
            imageStyle.width = originalWidth;
        }
        imageProps.style = imageStyle;

        return (
            <NativeImage { ...imageProps } />
        );
    };

    // Generate container props
    const containerProps = {
        style: styles.container,
    };
    if (testID) { containerProps.testID = `${testID}__container`; }
    if (disabled) { containerProps.disabled = disabled; }
    if (!disabled && onLongPress) { containerProps.onLongPress = onLongPress; }
    if (!disabled && onPress) { containerProps.onPress = onPress; }

    // Generate indicator props
    const indicatorProps = {
        style: {
            width,
            height,
        },
        color: loadingIndicator.color,
        size: loadingIndicator.size,
    };
    if (testID) { indicatorProps.testID = `${testID}__indicator`; }

    return (
        <Fragment>
            { showLoadingIndicator &&
                <ActivityIndicator { ...indicatorProps } />
            }
            { !showLoadingIndicator &&
                <Ripple { ...containerProps }>
                    { imageSrc ? renderImage() : renderPlaceholder() }
                </Ripple>
            }
        </Fragment>
    );
};

Image.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the image */
        container: PropTypes.object,

        /** Styles for the image */
        image: PropTypes.object,

        /** Styles for the loading indicator */
        loadingIndicator: PropTypes.object,
    }),

    /** Disables image touch */
    disabled: PropTypes.bool,

    /** Height of the image */
    height: PropTypes.number,

    /** The image source (either a remote URL or a local file resource) */
    imageSrc: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.object,
    ]),

    /**
     * Called when the image was long pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onLongPress: PropTypes.func,

    /**
     * Called when the image was pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onPress: PropTypes.func,

    /** The color of the placeholder icon */
    placeholderIconColor: PropTypes.string,

    /** The text name of the placeholder icon */
    placeholderIconName: PropTypes.string,

    /** The size of the placeholder icon */
    placeholderIconSize: PropTypes.number,

    /** Controls the loading indicator visibility */
    showLoadingIndicator: PropTypes.bool,

    /**  Scale width and height independently, this may change the aspect ratio of the src */
    stretch: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Width of the image */
    width: PropTypes.number,
};

Image.defaultProps = {
    contextStyles: {},
    height: 25,
    placeholderIconName: 'file-image',
    placeholderIconSize: 125,
    placeholderIconColor: '#006e96',
    width: 25,
};

export default Image;
