import React from 'react';
import PropTypes from 'prop-types';
import { View, TouchableHighlight } from 'react-native';
import Icon from 'react-native-vector-icons/MaterialCommunityIcons';
import TextLabel from '../TextLabel/TextLabel';

// Styles
import getStyles from './MenuItem.styles';

/**
 * A selectable option for a menu
 */
const MenuItem = (props) => {
    const {
        contextStyles,
        disabled,
        iconName,
        onLongPress,
        onPress,
        reverse,
        selected,
        testID,
        text,
    } = props;

    const styles = getStyles(contextStyles, reverse);

    // Generate touchable props
    const touchableProps = {};
    if (!disabled && onPress) { touchableProps.onPress = onPress; }
    if (!disabled && onLongPress) { touchableProps.onLongPress = onLongPress; }
    if (testID) { touchableProps.testID = `${testID}__touchable`; }

    // Generate container props
    const containerProps = {
        style: selected ? styles.selectedContainer : styles.container,
    };
    if (testID) { containerProps.testID = `${testID}__container`; }

    // Generate icon props
    const iconProps = {
        name: iconName,
        style: selected ? styles.selectedIcon : styles.icon,
    };
    if (testID) { iconProps.testID = `${testID}__icon`; }

    // Generate text props
    const textProps = {
        contextStyles: {
            text: selected ? styles.selectedText : styles.text,
        },
    };
    if (testID) { textProps.testID = `${testID}__text`; }

    return (
        <TouchableHighlight { ...touchableProps }>
            <View { ...containerProps }>
                { !!iconName && !reverse &&
                <Icon { ...iconProps } />
                }
                { text &&
                <TextLabel { ...textProps }>
                    { text }
                </TextLabel>
                }
                { !!iconName && reverse &&
                <Icon { ...iconProps } />
                }
            </View>
        </TouchableHighlight>
    );
};

MenuItem.defaultProps = {
    contextStyles: {},
};

MenuItem.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the text */
        container: PropTypes.object,

        /** Styles for the icon */
        icon: PropTypes.object,

        /** Styles for the text */
        text: PropTypes.object,
    }),

    /** Disable item press events */
    disabled: PropTypes.bool,

    /** Icon to display */
    iconName: PropTypes.string,

    /**
     * Called when the item is long pressed
     * @param {SyntheticEvent} event - The native long press event
     */
    onLongPress: PropTypes.func,

    /**
     * Called when the item is pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onPress: PropTypes.func,

    /** Reverse display of text and icon */
    reverse: PropTypes.bool,

    /** Toggle selected state */
    selected: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Text to display */
    text: PropTypes.string,
};

export default MenuItem;
