import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { UIManager, SafeAreaView, View, ScrollView, Modal as NativeModal, TouchableWithoutFeedback, StyleSheet } from 'react-native';

// Styles
import getStyles from './Modal.styles';

/**
 * A modal that displays content in the middle of the entire viewport and over all other content.
 */
class Modal extends Component {
    static propTypes = {
        /** Content to display within the modal */
        children: PropTypes.node.isRequired,

        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the overlay */
            overlay: PropTypes.object,

            /** Styles for the modal container */
            modal: PropTypes.object,
        }),

        /** Enables the modal to fully cover the viewport, up to the full screen ratio, instead of sizing to its contents */
        fullScreen: PropTypes.bool,

        /**
         * Called after the modal dimensions were applied
         * @param {{ width: Number, height: Number, top: Number, left: Number }} modalMeasurements - The dimensions of the modal
         * @param {{ width: Number, height: Number, top: Number, left: Number }} containerMeasurements - The dimensions of the modal and overlay container
         */
        onAfterPositionModal: PropTypes.func,

        /**
         * Called before modal dimensions are applied and returns new modal dimensions
         * @param {{ width: Number, height: Number, top: Number, left: Number }} modalMeasurements - The dimensions of the modal
         * @param {{ width: Number, height: Number, top: Number, left: Number }} containerMeasurements - The dimensions of the modal and overlay container
         * @returns {{ width: Number, height: Number, top: Number, left: Number }}
         */
        onBeforePositionModal: PropTypes.func,

        /**
         * Called after the modal closes
         */
        onClose: PropTypes.func,

        /**
         * Close modal on overlay press
         * @param {SyntheticEvent} event - The native press event
         */
        overlayClose: PropTypes.bool,

        /** Id used for testing */
        testID: PropTypes.string,

        /** Changes the overlay darkened fill to transparent */
        transparent: PropTypes.bool,

        /** Modal visibility */
        visible: PropTypes.bool,
    };

    static defaultProps = {
        contextStyles: {},
    };

    constructor(props) {
        super(props);
        const { visible } = props;
        this.state = {
            // Tracks modal size and position
            modalDimensions: {},

            // Hides modal while modal dimensions are calculated
            modalOpacity: 0,

            visible,
        };
    }

    render() {
        const {
            children,
            contextStyles,
            testID,
            transparent,
        } = this.props;

        const {
            modalDimensions,
            modalOpacity,
            visible,
        } = this.state;

        const styles = getStyles(contextStyles, transparent);

        const modalStyles = {
            ...styles.modal,
            ...modalDimensions,
            opacity: modalOpacity,
        };

        return (
            <NativeModal
                visible={ visible }
                onRequestClose={ this.handleRequestCloseModal }
                supportedOrientations={ [
                    'portrait',
                    'landscape',
                ] }
                transparent>

                <SafeAreaView
                    style={ { flex: 1 } }
                    testID={ testID }>

                    <View
                        style={ { flex: 1 } }
                        onLayout={ this.handleLayoutContainer }>

                        <TouchableWithoutFeedback onPress={ this.handlePressOverlay }>
                            <View style={ [
                                StyleSheet.absoluteFill,
                                styles.overlay,
                            ] } />
                        </TouchableWithoutFeedback>

                        <View style={ modalStyles }>
                            <ScrollView style={ { flex: 1 } }>
                                <View onLayout={ this.handleLayoutModal }>
                                    { children }
                                </View>
                            </ScrollView>
                        </View>

                    </View>

                </SafeAreaView>

            </NativeModal>
        );
    }

    componentDidUpdate(prevProps) {
        const { visible: wasVisible } = prevProps;
        const { visible: isVisible } = this.props;
        if (isVisible !== wasVisible) {
            this.reset({ visible: isVisible });
        }
    }

    /**
     * Saves the wrapping container for dimension calculation purposes
     * @param {LayoutEvent} event - The native layout event
     */
    handleLayoutContainer = (event) => {
        const { target } = event;
        this.setState({
            container: target,
        }, () => { this.measureContents(); });
    };

    /**
     * Saves the modal container for dimension calculation purposes
     * @param {LayoutEvent} event - The native layout event
     */
    handleLayoutModal = (event) => {
        const { target } = event;
        this.setState({
            modal: target,
        }, () => { this.measureContents(); });
    };

    /**
     * Called when the modal is closed
     */
    handleRequestCloseModal = () => {
        this.close();
    };

    /**
     * Handle overlay press
     */
    handlePressOverlay = () => {
        const { overlayClose } = this.props;
        if (overlayClose) {
            this.close();
        }
    };

    /**
     * Close the modal
     */
    close = () => {
        const { onClose } = this.props;
        this.setState({
            visible: false,
        }, () => {
            if (onClose) {
                onClose();
            }
        });
    };

    /**
     * Measures the modal and wrapping container
     */
    measureContents = () => {
        const {
            container, modal,
        } = this.state;

        if (container && modal) {
            const measureContainer = new Promise((resolve) => {
                UIManager.measure(container, (x, y, width, height) => {
                    resolve({
                        width,
                        height,
                    });
                });
            });

            const measureModal = new Promise((resolve) => {
                UIManager.measure(modal, (x, y, width, height) => {
                    resolve({
                        width,
                        height,
                    });
                });
            });

            Promise.all([
                measureContainer,
                measureModal,
            ])
                .then((dataArray) => {
                    const containerMeasurements = dataArray[0];
                    const modalMeasurements = dataArray[1];
                    this.calculateModalDimensions(containerMeasurements, modalMeasurements);
                });
        }
    };

    /**
     * Calculates modal dimension and position based on props and contents
     * @param {{ width: Number, height: Number, top: Number, left: Number }} containerMeasurements - The dimensions of the modal and overlay container
     * @param {{ width: Number, height: Number, top: Number, left: Number }} modalMeasurements - The dimensions of the modal
     */
    calculateModalDimensions = (containerMeasurements, modalMeasurements) => {
        const {
            width: windowWidth, height: windowHeight,
        } = containerMeasurements;
        const {
            width: modalWidth, height: modalHeight,
        } = modalMeasurements;

        const {
            fullScreen,
            onBeforePositionModal,
        } = this.props;

        // Set maximum modal size
        const maxModalHeight = windowHeight;
        const maxModalWidth = windowWidth;

        // Prevent modal from exceeding window size
        const newModalHeight = modalHeight < windowHeight ? modalHeight : windowHeight;
        const newModalWidth = modalWidth < windowWidth ? modalWidth : windowWidth;

        // Auto size modal to its content by default, centering it in the window
        let modalDimensions = {
            width: newModalWidth,
            height: newModalHeight,
            top: (windowHeight - newModalHeight) / 2,
            left: (windowWidth - newModalWidth) / 2,
        };

        // Override if full screen
        if (fullScreen) {
            modalDimensions = {
                width: windowWidth,
                height: windowHeight,
                top: (windowHeight - maxModalHeight) / 2,
                left: (windowWidth - maxModalWidth) / 2,
            };
        }

        // Handle modification of modal dimensions if supplied
        if (onBeforePositionModal) {
            Promise.resolve(onBeforePositionModal(modalDimensions, containerMeasurements))
                .then((updatedModalDimensions) => {
                    this.setModalDimensions(updatedModalDimensions, containerMeasurements);
                });
        }
        else {
            this.setModalDimensions(modalDimensions, containerMeasurements);
        }
    };

    /**
     * Sets modal dimensions
     * @param {{ width: Number, height: Number, top: Number, left: Number }} modalDimensions - The dimensions of the modal
     * @param {{ width: Number, height: Number, top: Number, left: Number }} containerMeasurements - The dimensions of the modal and overlay container
     */
    setModalDimensions = (modalDimensions, containerMeasurements) => {
        const {
            onAfterPositionModal,
        } = this.props;

        this.setState({
            modalDimensions,
            modalOpacity: 1,
        }, () => {
            if (onAfterPositionModal) {
                onAfterPositionModal(modalDimensions, containerMeasurements);
            }
        });
    };

    /**
     * Handles state updates after componentDidUpdate
     * @param {Object} newState
     */
    reset = (newState) => {
        this.setState(newState);
    };
}

export default Modal;
