import React from 'react';
import PropTypes from 'prop-types';
import ChartWrapper from '../ChartWrapper/ChartWrapper';

import getStyles from './PieChart.styles';

/**
 * The pie chart is mainly used for showing proportion of different categories.
 * Each arc length represents the proportion of data quantity.
 * @see https://echarts.apache.org/examples/en/editor.html?c=pie-simple
 */
const PieChart = (props) => {
    const {
        animation,
        contextStyles,
        horizontal,
        legend,
        onPress,
        seriesData,
        testID,
        title,
        tooltip,
    } = props;

    const styles = getStyles(contextStyles);

    /** Build chart options */
    const chartOptions = {
        animation,
        backgroundColor: 'transparent',
        color: [ ...ChartWrapper.defaultColors ],
        series: {
            // Default to vertical layout with pie centered
            center: [
                '50%',
                '50%',
            ],

            // Suppress slice labels due to real estate concerns
            label: { show: false },

            // Suppress label to slice lines as well
            labelLine: { show: false },

            // This is a pie chart
            type: 'pie',

            // Size it appropriately
            radius: '75%',
        },
    };

    // Set chart legend
    if (legend) {
        // Adjust chart center
        if (horizontal) {
            chartOptions.series.center[0] = '35%';
        }

        // Set chart legend layout
        const legendLocation = {
            right: horizontal ? '20%' : '5%',
            bottom: '10%',
            top: '10%',
            left: horizontal ? 'auto' : '5%',
            orient: horizontal ? 'vertical' : 'horizontal',
            type: 'scroll',
        };

        chartOptions.legend = {
            // Show the legend
            show: true,

            // Set legend series names
            data: seriesData.map((item) => (item.label)),

            // Set legend layout
            ...legendLocation,
        };
    }

    // Set chart title
    if (title) {
        chartOptions.title = {
            text: title,
            x: 'center',
        };
    }

    // Enable chart tooltip
    if (tooltip) {
        chartOptions.tooltip = {
            confine: true,
        };
    }

    // Generate chart series data
    chartOptions.series.data = seriesData.map((sliceData) => {
        const {
            borderColor,
            borderWidth,
            color,
            emphasisColor,
            label,
            opacity,
            selected,
            value,
            ...sliceRest
        } = sliceData;

        // Begin slice detail
        const pieSlice = {
            // Initialize slice style
            itemStyle: {},

            // Tag the slice with a name
            name: label,

            // Slice selected state
            selected,

            // Slice value
            value,

            // Pass through any extra props
            ...sliceRest,
        };

        // Add slice details
        if (borderColor) {
            pieSlice.itemStyle.borderColor = borderColor;
        }
        if (borderWidth || borderWidth === 0) {
            pieSlice.itemStyle.borderWidth = borderWidth;
        }
        if (color) {
            pieSlice.itemStyle.color = color;
        }
        if (opacity || opacity === 0) {
            pieSlice.itemStyle.opacity = opacity;
        }
        if (emphasisColor) {
            pieSlice.emphasis = { itemStyle: { color: emphasisColor } };
        }

        return pieSlice;
    });

    // Generate props
    const chartProps = {
        contextStyles: {
            container: styles.container,
        },
        option: chartOptions,
        canvas: true,
    };
    if (onPress) { chartProps.onPress = onPress; }
    if (testID) { chartProps.testID = testID; }

    return (
        <ChartWrapper { ...chartProps } />
    );
};

PieChart.propTypes = {
    /** Enables/disables chart animations */
    animation: PropTypes.bool,

    /** Styles for the component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,
    }),

    /** Display horizontal with legend on right */
    horizontal: PropTypes.bool,

    /** Displays the information for the charts */
    legend: PropTypes.bool,

    /**
     * Triggered on press of a pie section
     * @param {Object} itemData - Data for the pie section pressed
     */
    onPress: PropTypes.func,

    /** The data for the pie chart */
    seriesData: PropTypes.arrayOf(PropTypes.shape({
        /** Border color of the pie section */
        borderColor: PropTypes.string,

        /** Border width of the pie section */
        borderWidth: PropTypes.number,

        /** Color of the pie section */
        color: PropTypes.string,

        /** Highlight color of the pie on press/hover */
        emphasisColor: PropTypes.string,

        /** Name of the sliced data */
        label: PropTypes.string.isRequired,

        /** Opacity of the pie section */
        opacity: PropTypes.number,

        /** Selected state of the slice */
        selected: PropTypes.bool,

        /** The value of the pie section */
        value: PropTypes.number.isRequired,
    })).isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text for the chart */
    title: PropTypes.string,

    /** Show related data in a tooltip after pressing the pie section */
    tooltip: PropTypes.bool,
};

PieChart.defaultProps = {
    animation: true,
    legend: true,
    contextStyles: {},
    tooltip: true,
};

export default PieChart;
