import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { View, ActivityIndicator } from 'react-native';

// Styles
import getStyles from './RNTable.styles';

class RNTable extends Component {
    static propTypes = {
        /** Styles applied to every cell */
        cellStyle: PropTypes.oneOfType([
            PropTypes.object,
            PropTypes.arrayOf(PropTypes.object),
        ]),

        /** Information for each column cell */
        columnModel: PropTypes.arrayOf(PropTypes.shape({
            /** Enable the column to wrap its contents */
            wrap: PropTypes.bool,
        })),

        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the main container */
            container: PropTypes.object,

            /** Styles for each row container */
            row: PropTypes.object,

            /** Styles for each column container */
            column: PropTypes.object,
        }),

        /**
         * Multidimensional array of cells
         *  [  // rows
         *      [  // row
         *          e1,  // column
         *          e2,  // column
         *          e3,  // column
         *      ],
         *      [ e4, e5 ],
         *      [ e6, e7, e8 ],
         *  ]
         */
        rows: PropTypes.arrayOf(PropTypes.arrayOf(PropTypes.element)).isRequired,

        /** Styles applied to every row */
        rowStyle: PropTypes.oneOfType([
            PropTypes.object,
            PropTypes.arrayOf(PropTypes.object),
        ]),
    };

    static defaultProps = {
        contextStyles: {},
    };

    constructor(props) {
        super(props);

        // Initialize state
        this.state = {
            // Stores the flag used to resize columns after initial render
            // No need to resize layout for compact style
            resizeLayout: true,
        };

        // Stores the original column widths
        this.initialColumnSizes = [];

        // Stores the number of columns
        this.columnCount = 0;

        // Stores the minimum widths used to resize columns after initial render
        this.columnSizeMinimums = [];

        // Initialize total number of cells counter
        this.totalCells = 0;

        // Initialize total number of cells that have had their natural dimensions recorded
        this.sizedCells = 0;
    }

    render() {
        const {
            contextStyles,
        } = this.props;
        const {
            resizeLayout,
        } = this.state;

        // Generate component styles
        this.styles = getStyles(contextStyles);

        // Reset total cells counter as it gets incremented on every row column cell render
        // This is kept track of to determine when all cell layout events have been fired
        this.totalCells = 0;

        // Generate container props
        const containerProps = {
            style: {
                ...this.styles.container,

                // Prevent Flash Of Un-styled Content
                // Initially set the container opacity to zero so that elements render
                // and cell layout events fire but it is invisible to the user.
                // As cell layout events are fired the cell sizes are collected.
                // When the layout events are completed the column widths are determined and
                // a state update is made to re-render the table with the desired column sizes
                // and set the opacity back to one to show the sized table to the user.
                opacity: resizeLayout ? 0 : 1,
            },
        };

        // Render layout container and contents
        return (
            <Fragment>
                { resizeLayout && <ActivityIndicator /> }
                <View { ...containerProps }>
                    { this.renderRows() }
                </View>
            </Fragment>
        );
    }

    componentDidUpdate(prevProps) {
        const { rows: oldRows } = prevProps;
        const { rows: newRows } = this.props;
        if (oldRows !== newRows) {
            this.initialColumnSizes = [];
            this.columnCount = 0;
            this.columnSizeMinimums = [];
            this.totalCells = 0;
            this.sizedCells = 0;
            this.reset({
                resizeLayout: true,
            });
        }
    }

    /**
     * Renders a 'row' container for every item in the supplied array
     * @return {Array<React.ElementType>}
     */
    renderRows = () => {
        const {
            rows,
            rowStyle,
        } = this.props;
        const {
            row: rowStyles,
        } = this.styles;

        // Generate row props
        const rowProps = {
            style: {
                ...rowStyles,
                ...rowStyle,
            },
        };

        // Render row with its columns and cell contents
        return rows.map((columns, rowIndex) => (
            /* eslint-disable react/no-array-index-key */
            <View
                { ...rowProps }
                key={ `row_${rowIndex}` }>
                { this.renderColumns(columns, rowIndex) }
            </View>
            /* eslint-disable react/no-array-index-key */
        ));
    };

    /**
     * Renders a 'column cell' container for every item in the supplied array.
     *  column cells get a layout callback to collect their natural dimensions
     *  collected column dimensions are used, if available, to size column cells in a table-like format
     * @param {Array} columns
     * @param {Number} rowIndex
     * @return {Array<React.ElementType>}
     */
    renderColumns = (columns = [], rowIndex) => {
        const {
            cellStyle,
            columnModel,
        } = this.props;
        const {
            resizeLayout,
        } = this.state;
        const {
            column: columnStyles,
        } = this.styles;

        return columns.map((cell, columnIndex) => {
            // Increase recorded total cell count to help determine
            // when all cells have had their layouts calculated
            this.totalCells += 1;

            // Begin column props
            const columnProps = {
                style: {
                    ...columnStyles,
                    ...cellStyle,
                },
            };

            // If resizing the layout
            if (resizeLayout) {
                // Add layout prop to column
                columnProps.onLayout = (event) => {
                    // Map layout to array of columns and rows
                    this.handleCellLayout(event, rowIndex, columnIndex);
                };

                columnProps.style.flex = 0;
            }

            // Column sizing completed
            else if (
                // If last column
                columnIndex === this.columnCount - 1 ||

                // Or if set to wrap
                (
                    columnModel &&
                    columnModel[columnIndex] &&
                    columnModel[columnIndex].wrap
                )
            ) {
                // Flex to take up rest of space
                columnProps.style.flex = 1;
            }

            // Column sizing completed, add sizes to column
            else {
                columnProps.style.width = this.columnSizeMinimums[columnIndex];
            }

            // Render column with its cell contents
            return (
                <View
                    { ...columnProps }
                    key={ `column_${columnIndex}_${this.totalCells}` }>
                    { cell }
                </View>
            );
        });
    };

    /**
     * Gathers natural cell dimensions and counts number of columns
     * @param {LayoutEvent} event
     * @param {Number} rowIndex
     * @param {Number} columnIndex
     */
    handleCellLayout = (event, rowIndex, columnIndex) => {
        const { nativeEvent } = event;
        const { layout } = nativeEvent;
        const { width } = layout;

        // Shift zero-based index to indicate human readable column number
        const columnCount = columnIndex + 1;

        // Ensure array value exists (no undefined)
        if (!this.initialColumnSizes[rowIndex]) {
            this.initialColumnSizes[rowIndex] = [];
        }

        // Populate natural column content size
        this.initialColumnSizes[rowIndex][columnIndex] = width;

        // Increase column count if current column number is greater than previously recorded
        if (this.columnCount < columnCount) {
            this.columnCount = columnCount;
        }

        // Increase count of number of cells that have had their natural dimensions recorded
        // due to async nature of the cell element onLayout calls
        this.sizedCells += 1;

        // If collecting all cell dimensions is completed
        if (this.totalCells === this.sizedCells) {
            // Filter minimum required sizes and re-render
            this.generateColumnSizes();
        }
    };

    /**
     * Filters natural column sizes to the minimum required and forces a re-render
     * to redraw columns with calculated minimum dimensions
     */
    generateColumnSizes = () => {
        // For each column
        for (let colIdx = 0; colIdx < this.columnCount; colIdx += 1) {
            // For each row
            for (let rowIdx = 0; rowIdx < this.initialColumnSizes.length; rowIdx += 1) {
                // Get the natural cell size
                const columnSize = (this.initialColumnSizes[rowIdx] && this.initialColumnSizes[rowIdx][colIdx]) || 0;

                // Ensure array value exists (no undefined)
                if (!this.columnSizeMinimums[colIdx]) {
                    this.columnSizeMinimums[colIdx] = 0;
                }

                // Increase minimum column width if current content is larger than previously recorded content
                if (this.columnSizeMinimums[colIdx] < columnSize) {
                    this.columnSizeMinimums[colIdx] = columnSize;
                }
            }
        }

        // Re-render so that new sizes can be used to reconstruct cells but prevent further layout updates
        this.setState({
            resizeLayout: false,
        });
    };

    reset = (newState) => {
        this.setState(newState);
    };
}

export default RNTable;
