import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { View, Keyboard, TextInput } from 'react-native';

// Styles
import getStyles from './TextField.styles';

/**
 * A component that allows for the entry of text.
 */
class TextField extends PureComponent {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container surrounding the label and radio button */
            container: PropTypes.object,

            /** Styles for the text field */
            input: PropTypes.object,
        }),

        /** Sets automatic capitalization of certain characters */
        autoCapitalize: PropTypes.oneOf([
            'characters',
            'none',
            'sentences',
            'words',
        ]),

        /** Enables auto-correct */
        autoCorrect: PropTypes.bool,

        /** Focuses the input */
        autoFocus: PropTypes.bool,

        /** Grows/shrinks the input based on the size of its text value */
        autoGrow: PropTypes.bool,

        /** Enable the text field to blur when submitted */
        blurOnSubmit: PropTypes.bool,

        /** Prevents the text field from accepting input */
        disabled: PropTypes.bool,

        /** Determines which keyboard to open */
        keyboardType: PropTypes.oneOf([
            'decimal-pad',
            'default',
            'email-address',
            'number-pad',
            'numeric',
            'phone-pad',
            'visible-password',
        ]),

        /** Limits the maximum number of characters that can be entered */
        maxLength: PropTypes.number,

        /** Enables the text input to be multiple lines */
        multiline: PropTypes.bool,

        /** Called when the text input is blurred */
        onBlur: PropTypes.func,

        /**
         * Called when the text input's text changes
         *
         * @param {String} value - The new text value
         */
        onChangeText: PropTypes.func,

        /**
         * Called when the text input's content size changes
         *
         * @param {{ nativeEvent: { contentSize: { width: Number, height: Number } } }} event - The native content change event
         */
        onContentSizeChange: PropTypes.func,

        /**
         * Called when the text input is focused
         *
         * @param {{ nativeEvent: { target: Object } }} event - The native focus event
         */
        onFocus: PropTypes.func,

        /**
         * Called when the text input's submit button is pressed
         *
         * @param {Object} componentInstance - This component instance
         * @param {Object} componentRef - The internal text input reference
         * @param {{ nativeEvent: { text: String, eventCount: Number, target: Object } }} event - The native submit event
         */
        onSubmitEditing: PropTypes.func,

        /** The string that will be rendered before text input has been entered */
        placeholder: PropTypes.string,

        /** Determines how the return key should look */
        returnKeyType: PropTypes.oneOf([
            'default',
            'done',
            'go',
            'next',
            'search',
            'send',
        ]),

        /** Enables the text input to obscure the text entered so that sensitive text like passwords stay secure */
        secureTextEntry: PropTypes.bool,

        /** Enables automatic selection of all text on input focus */
        selectTextOnFocus: PropTypes.bool,

        /** Id used for testing */
        testID: PropTypes.string,

        /** Set the Android input underline color */
        underlineColorAndroid: PropTypes.string,

        /** The text to populate the input */
        value: PropTypes.string,
    };

    static defaultProps = {
        contextStyles: {},
        keyboardType: 'default',
        returnKeyType: 'done',
        autoCapitalize: 'none',
    };

    constructor(props) {
        super(props);
        const { value } = props;
        this.state = {
            text: value,
            textWidth: 24,
            inputMaxWidth: 0,
            isInputMeasured: false,
        };
    }

    render() {
        const {
            autoGrow,
            contextStyles,
            disabled,
            testID,
        } = this.props;

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: disabled ? styles.disabledContainer : styles.container,
        };
        if (testID) { containerProps.testID = `${testID}__container`; }

        // Generate input props
        const inputProps = this.getInputProps(styles);
        if (testID) { inputProps.testID = `${testID}__input`; }

        return (
            <View { ...containerProps }>

                { autoGrow ?
                    <View
                        style={ this.getAutoGrowContainerStyles() }
                        onLayout={ this.handleLayout }>
                        <TextInput { ...this.getAutoGrowInputProps(inputProps) } />
                    </View>
                    :
                    <TextInput { ...inputProps } />
                }

            </View>
        );
    }

    componentDidUpdate(prevProps) {
        // Ensure the text gets updated on prop changes
        const { value } = this.props;
        if (prevProps.value !== value) {
            this.handleChangeText(value);
        }
    }

    /**
     * Called when the text input is blurred
     */
    handleBlur = () => {
        const { onBlur } = this.props;
        if (onBlur) {
            onBlur(this.textField.props.value);
        }
    };

    /**
     * Called when the text input's text changes
     *
     * @param {String} text - The new text value
     */
    handleChangeText = (text) => {
        const {
            onChangeText,
            disabled,
        } = this.props;
        if (!disabled) {
            this.setState({ text }, () => {
                if (onChangeText) {
                    onChangeText(text);
                }
            });
        }
    };

    /**
     * Called when the text input's content size changes
     *
     * @param {{ nativeEvent: { contentSize: { width: Number, height: Number } } }} event - The native content change event
     */
    handleContentSizeChange = (event) => {
        const { onContentSizeChange } = this.props;
        const { nativeEvent = {} } = event;
        const { contentSize = {} } = nativeEvent;
        const { width: textWidth = 0 } = contentSize;
        this.setState({ textWidth }, () => {
            if (onContentSizeChange) {
                onContentSizeChange(event);
            }
        });
    };

    /**
     * Called when the text input is focused
     *
     * @param {{ nativeEvent: { target: Object } }} event - The native focus event
     */
    handleFocus = (event) => {
        const { onFocus } = this.props;
        if (onFocus) {
            onFocus(event);
        }
    };

    /**
     * Called when the text input's submit button is pressed
     *
     * @param {{ nativeEvent: { text: String, eventCount: Number, target: Object } }} event - The native submit event
     */
    handleSubmitEditing = (event) => {
        const { onSubmitEditing } = this.props;
        if (onSubmitEditing) {
            onSubmitEditing(this, this.textField, event);
        }
        else {
            Keyboard.dismiss();
        }
    };

    /**
     * Sets the input focus
     *
     * @public
     */
    setFocus = () => {
        this.textField.focus();
    };

    /**
     * Generate base input properties
     *
     * @param {Object} styles
     * @return {Object}
     */
    getInputProps = (styles) => {
        const {
            autoCapitalize,
            autoCorrect,
            autoFocus,
            blurOnSubmit,
            disabled,
            keyboardType,
            maxLength,
            multiline,
            placeholder,
            returnKeyType,
            secureTextEntry,
            selectTextOnFocus,
            underlineColorAndroid,
        } = this.props;

        const {
            text,
        } = this.state;

        return {
            autoCapitalize,
            autoCorrect,
            autoFocus,
            blurOnSubmit,
            editable: !disabled,
            keyboardType,
            maxLength,
            multiline,
            onLayout: this.handleLayout,
            onBlur: this.handleBlur,
            onChangeText: this.handleChangeText,
            onContentSizeChange: this.handleContentSizeChange,
            onFocus: this.handleFocus,
            onSubmitEditing: this.handleSubmitEditing,
            placeholder,
            returnKeyType,
            secureTextEntry,
            selectTextOnFocus,
            underlineColorAndroid,
            value: text,
            style: styles.input,
            ref: (ref) => {
                this.textField = ref;
            },
        };
    };

    /**
     * Generate auto grow input properties
     *
     * @param {Object} inputProps
     * @return {Object}
     */
    getAutoGrowInputProps = (inputProps) => {
        const {
            textWidth,
            inputMaxWidth,
        } = this.state;

        // Extract the input font size
        const { style = {} } = inputProps;
        const { input = {} } = style;
        const { fontSize = 16 } = input;

        // Calculate a minimum input width to prevent an unusable size
        const minimumInputWidth = fontSize * 2 || 32;

        // Calculate the auto grow input width
        const inputWidth = textWidth + ((fontSize * 2) || 32);

        // Define auto grow input props
        return {
            ...inputProps,

            // Disable multiline as this is not compatible with auto grow
            multiline: false,

            // Build new styles for auto grow input
            style: {
                // Start with existing style
                ...inputProps.style,

                // Prevent the input from shrinking down to an unusable size
                minWidth: minimumInputWidth,

                // Prevent the input from growing past its wrapping container
                maxWidth: inputMaxWidth,

                // Set the input width to the size of the typed text (plus a little padding for aesthetics)
                width: inputWidth,

                // Prevent the input from bumping into the edge of its wrapping container
                paddingRight: 8,

                // Disable the flexing of the input so that hard measurements get used
                flex: 0,

                // For testing
                // borderWidth: 1,
                // borderColor: '#00f',
            },
        };
    };

    /**
     * Generate styles for auto grow input container
     *
     * @return {Object}
     */
    getAutoGrowContainerStyles = () => {
        // Start with the wrapping container flexed to fill its wrapping container
        // in order to determine the maximum size the input can grow to
        return {
            flex: 1,

            // For testing
            // borderWidth: 1,
            // borderColor: '#f00',
        };
    };

    /**
     * Generate maximum input width
     *
     * @param {LayoutEvent} event
     */
    handleLayout = (event) => {
        const { isInputMeasured } = this.state;
        if (!isInputMeasured) {
            const { nativeEvent } = event;
            const { layout } = nativeEvent;
            const { width } = layout;
            this.setState({
                isInputMeasured: true,
                inputMaxWidth: width,
            });
        }
    };
}

export default TextField;
