import React from 'react';
import PropTypes from 'prop-types';
import { View, Text } from 'react-native';

// Styles
import getStyles from './TextLabel.styles';

/**
 * A component for displaying text.
 */
const TextLabel = (props) => {
    const {
        children,
        contextStyles,
        disabled,
        numberOfLines,
        onLongPress,
        onPress,
        selectable,
        selectionColor,
        suppressHighlighting,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    // Generate container props
    const containerProps = {
        style: styles.container,
    };
    if (testID) { containerProps.testID = `${testID}__container`; }

    // Generate text props
    const textProps = {
        children,
        style: styles.text,
    };
    if (numberOfLines) { textProps.numberOfLines = numberOfLines; }
    if (!disabled && onLongPress) { textProps.onLongPress = onLongPress; }
    if (!disabled && onPress) { textProps.onPress = onPress; }
    if (selectable) { textProps.selectable = selectable; }
    if (selectionColor) { textProps.selectionColor = selectionColor; }
    if (suppressHighlighting) { textProps.suppressHighlighting = suppressHighlighting; }
    if (testID) { textProps.testID = `${testID}__text`; }

    return (
        <View { ...containerProps }>
            <Text { ...textProps } />
        </View>
    );
};

TextLabel.propTypes = {
    /** The text to display */
    children: PropTypes.string.isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the text */
        container: PropTypes.object,

        /** Styles for the text */
        text: PropTypes.object,
    }),

    /** Disables text touch */
    disabled: PropTypes.bool,

    /**
     * Used to truncate the text with an ellipsis after
     * computing the text layout, including line wrapping,
     * such that the total number of lines does not exceed this number
     */
    numberOfLines: PropTypes.number,

    /**
     * Called when the text is long pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onLongPress: PropTypes.func,

    /**
     * Called when the text is pressed
     * @param {SyntheticEvent} event - The native press event
     */
    onPress: PropTypes.func,

    /** Enables the user to select text to use the native copy and paste functionality */
    selectable: PropTypes.bool,

    /** The highlight color of the text */
    selectionColor: PropTypes.string,

    /**
     * When true, no visual change is made when text is pressed down
     * By default, a gray oval highlights the text on press down
     */
    suppressHighlighting: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,
};

TextLabel.defaultProps = {
    contextStyles: {},
};

export default TextLabel;
