import React from 'react';
import PropTypes from 'prop-types';
import Ripple from 'react-native-material-ripple';
import TextLabel from '../TextLabel/TextLabel';
import Image from '../Image/Image';

// Styles
import getStyles from './TextWithImage.styles';

/**
 * A component for displaying text.
 */
const TextWithImage = (props) => {
    const {
        contextStyles,
        disabled,
        imageHeight,
        imageSrc,
        imageStretch,
        imageWidth,
        numberOfLines,
        onLongPress,
        onPress,
        placeholderIconColor,
        placeholderIconName,
        placeholderIconSize,
        reverse,
        selectable,
        selectionColor,
        showLoadingIndicator,
        stacked,
        suppressHighlighting,
        testID,
        text,
    } = props;

    const styles = getStyles(contextStyles, reverse, stacked);

    // Generate container props
    const containerProps = {
        style: styles.container,
    };
    if (disabled) { containerProps.disabled = disabled; }
    if (!disabled && onLongPress) { containerProps.onLongPress = onLongPress; }
    if (!disabled && onPress) { containerProps.onPress = onPress; }
    if (testID) { containerProps.testID = `${testID}__container`; }

    // Generate text props
    const textProps = {
        contextStyles: { text: styles.text },
    };
    if (numberOfLines) { textProps.numberOfLines = numberOfLines; }
    if (selectable) { textProps.selectable = selectable; }
    if (selectionColor) { textProps.selectionColor = selectionColor; }
    if (suppressHighlighting) { textProps.suppressHighlighting = suppressHighlighting; }
    if (testID) { textProps.testID = `${testID}__text`; }

    // Generate image props
    const imageProps = {
        contextStyles: {
            image: styles.image,
            loadingIndicator: styles.loadingIndicator,
        },
        height: imageHeight,
        imageSrc,
        width: imageWidth,
    };
    if (imageStretch) {
        imageProps.stretch = imageStretch;
        if (!stacked) {
            imageProps.contextStyles.container = {
                flex: 1,
                overflow: 'hidden',
            };
        }
    }
    if (placeholderIconColor) { imageProps.placeholderIconColor = placeholderIconColor; }
    if (placeholderIconName) { imageProps.placeholderIconName = placeholderIconName; }
    if (placeholderIconSize) { imageProps.placeholderIconSize = placeholderIconSize; }
    if (showLoadingIndicator) { imageProps.showLoadingIndicator = showLoadingIndicator; }
    if (testID) { imageProps.testID = `${testID}__image`; }

    return (
        <Ripple { ...containerProps }>
            <TextLabel { ...textProps }>
                { text }
            </TextLabel>
            <Image { ...imageProps } />
        </Ripple>
    );
};

TextWithImage.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the text and image */
        container: PropTypes.object,

        /** Styles for the image */
        image: PropTypes.object,

        /** Styles for the image loading indicator */
        loadingIndicator: PropTypes.object,

        /** Styles for the text */
        text: PropTypes.object,
    }),

    /** Disables image and text touch events */
    disabled: PropTypes.bool,

    /** Height of the image */
    imageHeight: PropTypes.number,

    /** The image source (either a remote URL or a local file resource) */
    imageSrc: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.object,
    ]).isRequired,

    /**  Scale width and height independently, this may change the aspect ratio of the src */
    imageStretch: PropTypes.bool,

    /** Width of the image */
    imageWidth: PropTypes.number,

    /**
     * Used to truncate the text with an ellipsis after
     * computing the text layout, including line wrapping,
     * such that the total number of lines does not exceed this number
     */
    numberOfLines: PropTypes.number,

    /**
     * Called when the text or image was long pressed
     * @param {Object} props - The component props
     * @param {SyntheticEvent} event - The native press event
     */
    onLongPress: PropTypes.func,

    /**
     * Called when the text or image was pressed
     * @param {Object} props - The component props
     * @param {SyntheticEvent} event - The native press event
     */
    onPress: PropTypes.func,

    /** The color of the image placeholder icon */
    placeholderIconColor: PropTypes.string,

    /** The text name of the image placeholder icon */
    placeholderIconName: PropTypes.string,

    /** The size of the image placeholder icon */
    placeholderIconSize: PropTypes.number,

    /** Reverse display of text and image */
    reverse: PropTypes.bool,

    /** Enables the user to select text to use the native copy and paste functionality */
    selectable: PropTypes.bool,

    /** The highlight color of the text */
    selectionColor: PropTypes.string,

    /** Controls the image loading indicator visibility */
    showLoadingIndicator: PropTypes.bool,

    /** Display text and image in a column */
    stacked: PropTypes.bool,

    /**
     * When true, no visual change is made when text is pressed down
     * By default, a gray oval highlights the text on press down
     */
    suppressHighlighting: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** The text to display */
    text: PropTypes.string.isRequired,
};

TextWithImage.defaultProps = {
    contextStyles: {},
};

export default TextWithImage;
