import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { View, Platform } from 'react-native';
import { Callout as MapCallout, CalloutSubview } from 'react-native-maps';
import uuidv4 from 'uuid/v4';

import Button from '../../Button/Button';
import Text from '../../TextLabel/TextLabel';

// Styles
import getStyles from './Callout.styles';

/**
 * A control for displaying information about a position on map when clicked on marker
 * @see https://github.com/react-native-community/react-native-maps/blob/master/docs/callout.md
 */
class Callout extends Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container */
            container: PropTypes.object,

            /** Styles for the button container */
            buttonContainer: PropTypes.object,

            /** Styles for the text containert */
            textContainer: PropTypes.object,
        }),

        /** Id of the marker which callout belongs to */
        id: PropTypes.string,

        /** Title text displayed inside the callout */
        title: PropTypes.string,

        /** Array of routes that contains information to display paths between selected points */
        routes: PropTypes.array,

        /**
         * Called when marker menu was selecte
         * @param {String} id - The selected marker id
         */
        onMarkerMenuPress: PropTypes.func,

        /** Button component for showing menu options */
        btnMenu: PropTypes.element,

        /** Id used for testing */
        testID: PropTypes.string,
    };

    static defaultProps = {
        title: '',
        id: '-1',
        routes: [],
        btnMenu: <Button text="menu" />,
    };

    constructor(props) {
        super(props);
        this.isAndroid = Platform.OS !== 'ios';
    }

    render() {
        const {
            title,
            contextStyles,
            testID,
            btnMenu,
        } = this.props;
        const styles = getStyles(contextStyles);

        // Generate Callout props
        const calloutProps = {
            tooltip: false,
            onPress: this.handleCalloutPress,
            testID,
        };

        if (this.isAndroid) {
            return (
                <MapCallout { ...calloutProps }>
                    <View
                        style={ {
                            ...styles.container,

                            // Need to give minheight as callout doesn't expand when content grows in android
                            minHeight: 125,
                        } }>
                        <View style={ styles.buttonContainer }>
                            { btnMenu }
                        </View>
                        <View style={ styles.textContainer }>
                            <Text numberOfLines={ 3 }>
                                { title }
                            </Text>
                            { this.renderRouteInformation() }
                        </View>
                    </View>
                </MapCallout>
            );
        }
        return (
            <MapCallout { ...calloutProps }>
                <View style={ styles.container }>
                    <CalloutSubview
                        style={ styles.buttonContainer }
                        onPress={ this.handleCalloutPress }>
                        { btnMenu }
                    </CalloutSubview>
                    <View style={ styles.textContainer }>
                        <Text numberOfLines={ 3 }>
                            { title }
                        </Text>
                        { this.renderRouteInformation() }
                    </View>
                </View>
            </MapCallout>
        );
    }

    /**
    * Renders route information on map
    */
    renderRouteInformation = () => {
        const { routes } = this.props;
        if (!routes || routes.length === 0) return null;

        return routes.map((route) => (
            <Text
                key={ uuidv4() }
                contextStyles={ { text: { color: route.color } } }>
                { `${route.leg.distance.text} / ${route.leg.duration.text}` }
            </Text>
        ));
    };

    /**
    * Handle Callout press
    */
    handleCalloutPress = () => {
        const {
            onMarkerMenuPress,
            id,
        } = this.props;
        if (onMarkerMenuPress) { onMarkerMenuPress(id); };
    };
};

export default Callout;
