import React from 'react';
import PropTypes from 'prop-types';
import { View } from 'react-native';
import MapView, { PROVIDER_GOOGLE } from 'react-native-maps';

import MapToggle from '../MapToggle/MapToggle';
import AddressSearch from '../AddressSearch/AddressSearch';
import getStyles from './Map.styles';

/**
 * A control for displaying maps.
 * @see https://github.com/react-native-community/react-native-maps/blob/master/docs/mapview.md
 */
class Map extends React.Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container */
            container: PropTypes.object,

            /** Styles for the map */
            map: PropTypes.object,
        }),

        /** Map Content */
        children: PropTypes.node,

        /** Fits the map to the pins when loaded */
        fitToPins: PropTypes.bool,

        /** Maximum number of routes to be displayed on map */
        maxNumberOfRoutes: PropTypes.number,

        /** Show user location */
        showsUserLocation: PropTypes.bool,

        /** Display my location button */
        showsMyLocationButton: PropTypes.bool,

        /** Follows the user location */
        followsUserLocation: PropTypes.bool,

        /** Region of the map */
        region: PropTypes.shape({
            latitude: PropTypes.number,
            longitude: PropTypes.number,
            latitudeDelta: PropTypes.number,
            longitudeDelta: PropTypes.number,
        }),

        /**
         * Called when the region was completed
         * @param {Object} region - The selected region
         */
        onRegionChangeComplete: PropTypes.func,

        /** Called when map was long pressed */
        onLongPress: PropTypes.func,

        /** Called when marker was pressed */
        onMarkerPress: PropTypes.func,

        /** Callback that is called once the map is fully loaded */
        onMapReady: PropTypes.func,

        /**
         * Called when the region is changed
         * @param {Object} region - The selected region
         */
        onRegionChange: PropTypes.func,

        /** Enable map search */
        enableSearch: PropTypes.bool,

        /** Enable map toggle button */
        enableMapToggle: PropTypes.bool,

        /**
        * Called when address is searched
        * @param {String} text - address text
        */
        onAddressSearch: PropTypes.func,

        /** Id used for testing */
        testID: PropTypes.string,
    };

    static defaultProps = {
        contextStyles: {},
        maxNumberOfRoutes: 3,
        fitToPins: false,
        showsUserLocation: true,
        showsMyLocationButton: true,
        followsUserLocation: true,
        region: null,
        enableMapToggle: true,
        enableSearch: true,
    };

    constructor(props) {
        super(props);
        this.mapRef = null;
        this.state = {
            showSatelliteView: false,
        };
    }

    render() {
        const {
            contextStyles,
            region,
            children,
            testID,
            fitToPins,
            showsUserLocation,
            maxNumberOfRoutes,
            showsMyLocationButton,
            onMapReady,
            followsUserLocation,
            onMarkerPress,
            onAddressSearch,
            enableSearch,
            enableMapToggle,
        } = this.props;

        const { showSatelliteView } = this.state;
        const mapStyles = getStyles(contextStyles);

        const mapProps = {
            style: mapStyles.map,
            moveOnMarkerPress: false,
            provider: PROVIDER_GOOGLE,
            mapType: showSatelliteView ? MapView.MAP_TYPES.SATELLITE : MapView.MAP_TYPES.STANDARD,
            fitToPins,
            showsUserLocation,
            maxNumberOfRoutes,
            showsMyLocationButton,
            region,
            onMapReady,
            followsUserLocation,
            onLongPress: this.handleOnLongPress,
            onMarkerPress,
            testID,
            onAddressSearch,
            onPanDrag: this.handleOnPanDrag,
            onRegionChangeComplete: this.handleOnRegionChangeComplete,
            onRegionChange: this.handleOnRegionChange,
        };

        const mapToggleProps = {
            provider: PROVIDER_GOOGLE,
            showSatelliteView: showSatelliteView,
            onToggleViewClick: this.handleOnToggleViewClick,
        };

        if (testID) { mapToggleProps.testID = `${testID}__toggle`; }

        return (
            <View style={ mapStyles.container }>
                <MapView
                    ref={ (el) => this.mapRef = el }
                    { ...mapProps }>
                    { children }
                </MapView>
                { enableMapToggle &&
                    <MapToggle
                        ref={ (el) => this.mapRefTile = el }
                        { ...mapToggleProps } /> }
                { enableSearch &&
                    <AddressSearch
                        onAddressSearch={ onAddressSearch } /> }
            </View>
        );
    }

    /**
     * Keep the toggle map in sync with the viewing map
     */
    setMapToggleCamera = () => {
        if (this.mapRef) {
            this.mapRef
                .getCamera()
                .then((camera) => {
                    if (this.mapRefTile){
                        this.mapRefTile.setCamera(camera);
                    }
                });
        }
    };

    /**
     * Fit to markers on the map
     * @param {Boolean} animated - Animate to range
     */
    fitToElements = (animated = false) => {
        if (!this.mapRef) { return; }

        this.mapRef.fitToElements(animated);

        // Need to set timeout so that toogle map can get the main map updated camera
        // since no proper event is getting fired to update toggle map
        const context = this;
        setTimeout(() => {
            context.setMapToggleCamera();
        }, 2000);
    };

    /**
     * Center map to coordinate
     * @param {Object} coordinate
     */
    animateToCoordinate = (coordinate) => {
        if (!this.mapRef) { return; }

        this.mapRef.animateCamera({ center: coordinate });

        // Need to set timeout so that toogle map can get the main map updated camera
        // since no proper event is getting fired to update toggle map
        const context = this;
        setTimeout(() => {
            context.setMapToggleCamera();
        }, 2000);
    };

    /**
     * Handle region change complete
     * @param {Object} region
     */
    handleOnRegionChangeComplete = (region) => {
        // Update toggle map to current camera view
        this.setMapToggleCamera();

        const { onRegionChangeComplete } = this.props;
        if (onRegionChangeComplete) { onRegionChangeComplete(region); }
    };

    /**
     * Handle region change
     * @param {Object} region
     */
    handleOnRegionChange = (region) => {
        // Update toggle map to current camera view
        this.setMapToggleCamera();
        const { onRegionChange } = this.props;
        if (onRegionChange) { onRegionChange(region); }
    };

    /**
     * Handle panning operations
     */
    handleOnPanDrag = () => {
        // Update toggle map to current camera view
        this.setMapToggleCamera();
    };

    /**
     * Handle toggle map click
     */
    handleOnToggleViewClick = () => {
        const { showSatelliteView } = this.state;
        this.setState({
            showSatelliteView: !showSatelliteView,
        });
    };

    /**
     * Handle when map is long pressed.
     * @param {Event} event - Map's Long Press event
     */
    handleOnLongPress = (event) => {
        const {
            onLongPress,
        } = this.props;
        const { nativeEvent: { coordinate } } = event;
        if (onLongPress) {
            onLongPress(coordinate);
        }
    };
}

export default Map;
