import React from 'react';
import PropTypes from 'prop-types';
import { View, TouchableOpacity } from 'react-native';
import MapView from 'react-native-maps';

import getStyles from './MapToggle.styles';

/**
 * A toogle control for switching between different map views.
 * @see https://github.com/react-native-community/react-native-maps/blob/master/docs/mapview.md
 */
class MapToggle extends React.Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container */
            container: PropTypes.object,

            /** Styles for the map toggle */
            mapToggle: PropTypes.object,
        }),

        /** Show satelite view */
        showSatelliteView: PropTypes.bool,

        /** Maps provider for toogle map */
        provider: PropTypes.string,

        /**
        * Called when the toggle view was pressed
        * @param {SyntheticEvent} event - The native press event
        */
        onToggleViewClick: PropTypes.func,

        /** Callback that is called once the map is fully loaded */
        onMapReady: PropTypes.func,

        /** Id used for testing */
        testID: PropTypes.string,
    };

    static defaultProps = {
        provider: null,
    };

    constructor(props) {
        super(props);
        this.mapRefTile = undefined;
    }

    render() {
        const {
            showSatelliteView,
            onToggleViewClick,
            provider,
            onMapReady,
            contextStyles,
            testID,
        } = this.props;
        const styles = getStyles(contextStyles);

        return (
            <View style={ styles.container }>
                <MapView

                    // Map instance reference to accessing the map API
                    ref={ (el) => {
                        this.mapRefTile = el;
                    } }

                    // Set the provider
                    provider={ provider }

                    // Standard / Satellite map
                    mapType={ showSatelliteView ? MapView.MAP_TYPES.STANDARD : MapView.MAP_TYPES.SATELLITE }

                    // Map container viewport styles
                    style={ styles.mapToogle }

                    testID={ testID }

                    // Notify map is ready
                    onMapReady={ onMapReady } />
                <TouchableOpacity

                    // Lets overlay this view so we can manage the click event. The google name is
                    // a click event and we don't want to jump to google when the name is clicked.
                    style={ styles.mapToogle }
                    onPress={ onToggleViewClick } />
            </View>
        );
    }

    /**
     * Set the camera position
     * @param {Object} camera - MapView camera object
     */
    setCamera = (camera) => {
        camera.zoom = camera.zoom - 2; // TODO: need to determine zoom ratio for this small screen.
        if (this.mapRefTile){
            this.mapRefTile.setCamera(camera);
        }
    };
}

export default MapToggle;