import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { View } from 'react-native';
import { Marker as MapMarker } from 'react-native-maps';

import Image from '../../Image/Image';
import getStyles from './Marker.styles';

/**
 * A control which renders a custon marker on a map
 */
class Marker extends React.Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container */
            container: PropTypes.object,

            /** Styles for the marker */
            marker: PropTypes.object,
        }),

        /** Marker details object */
        marker: PropTypes.shape({
            /** Id of the marker */
            id: PropTypes.string,

            /** Color of the marker */
            pinColor: PropTypes.string,

            /** Description of the marker */
            description: PropTypes.string,

            /** Name of the marker */
            name: PropTypes.string,

            /** Custom image link to be used as marker*/
            imageName: PropTypes.string,
        }),

        /** Marker was dropped by user */
        isDropped: PropTypes.bool,

        /**
        * Called when the marker is pressed.
        * @param {Object} coordinate - Marker coordinate
        * @param {String} id - Marker id
        */
        onPressMarker: PropTypes.func,

        /** Callback that is called once the marker is fully loaded */
        onMarkerRendered: PropTypes.func,

        /** Callback that is called once the marker is fully loaded */
        onLayout: PropTypes.func,

        /** Marker content */
        children: PropTypes.node,

        /** Marker zIndex */
        zIndex: PropTypes.number,

        /** Marker types */
        markerType: PropTypes.oneOf([
            'pin',
            'image',
        ]),

        /** Custom image height of marker. Applicable only when markerType is image.*/
        imgHeight: PropTypes.number,

        /** Custom image width of marker. Applicable only when markerType is image. */
        imgWidth: PropTypes.number,

        /** Id used for testing */
        testID: PropTypes.string,
    };

    static defaultProps = {
        marker: {},
        imgHeight: 40,
        imgWidth: 40,
    };

    render() {
        const {
            marker,
            isDropped,
            children,
            contextStyles,
            testID,
            imgHeight,
            imgWidth,
            markerType,
            zIndex,
        } = this.props;
        const coordinate = {
            latitude: marker.latitude,
            longitude: marker.longitude,
        };
        const styles = getStyles(contextStyles);
        const markerProps = {
            style: styles.marker,
            key: marker.id,
            coordinate: coordinate,
            pinColor: marker.pinColor,
            description: marker.description,
            title: marker.name,
            onPress: this.handleOnPress,
            isDropped: isDropped,
            onCalloutPress: this.handleOnMarkerMenuPress,
            zIndex,
            testID,
        };

        return (

            // NOTE: Wrapping this in a view because Marker does not call onLayout for Android.
            // onLayout is the only path that could be determined to know the Marker had successfully been added.
            <View
                onLayout={ this.handleOnLayout }
                style={ styles.container }>
                <MapMarker
                    ref={ (el) => this.markerRef = el }
                    { ...markerProps }>
                    <Fragment>
                        { markerType === 'image' &&
                            <Image
                                imageSrc={ { uri: marker.imageName } }
                                contextStyles={ styles.marker }
                                height={ imgHeight }
                                width={ imgWidth }
                                resizeMode="contain" />
                        }
                        { children }
                    </Fragment>
                </MapMarker>
            </View>
        );
    }

    /**
     * Handle when maker is pressed.
     * @param {Event} event - Marker's Press event
     */
    handleOnPress = (event) => {
        const {
            onPressMarker,
            marker,
        } = this.props;
        const { nativeEvent: { coordinate } } = event;
        if (onPressMarker) {
            onPressMarker(coordinate, marker.id);
        }
    };

    /**
     * Handle layout. This is currently the only way to know all markers are rendered
     * @param {SyntheticEvent} event - The native layout event
     */
    handleOnLayout = (event) => {
        const {
            onMarkerRendered,
            onLayout,
        } = this.props;
        if (onMarkerRendered) {
            // Hand out marker
            onMarkerRendered(this.markerRef);
        }

        if (onLayout) { onLayout(event); }
    };

    /**
     * Redraws the Callout
     */
    redrawCallout = () => {
        // Need to hide and show the callout since there is an internal bug with updating the callout when state changes.
        this.markerRef.hideCallout();
        this.markerRef.showCallout();
    }
}

Marker.MarkerType = {
    Image: 'image',
    Pin: 'pin',
};

export default Marker;