import React from 'react';
import PropTypes from 'prop-types';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import CircularProgress from '@material-ui/core/CircularProgress';

// Styles
import getStyles from './ActivityIndicator.styles';
import './style.css';

/** Default color of Activity Indicator */
export const ACTIVITY_INDICATOR_COLOR = 'inherit';

/** Default thickness of Activity Indicator */
export const ACTIVITY_INDICATOR_THICKNESS = 3.6; // default thickness as specified by Material UI.

/** Default size of Activity Indicator */
export const ACTIVITY_INDICATOR_SIZE = 40; // default size as specified by Material UI.

/**
 * Activity Indicator component :indicates the user that a control level action is currently
 * being performed and that further actions with that control should wait until the initial action is completed.
 * @see https://material-ui.com/demos/progress/#circular-indeterminate
 * @see https://material-ui.com/api/circular-progress/
 */
const ActivityIndicator = (props) => {
    const {
        contextStyles,
        thickness,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiCircularProgress: { // Name of the component ⚛️ / style sheet
                root: {// Name of the rule
                    ...styles.activityIndicator,
                },
            },
        },
    });

    /** The size for the activity indicator is being fetched from width/height from context styles. */
    let activityIndicatorSize = theme.overrides.MuiCircularProgress.root.width || theme.overrides.MuiCircularProgress.root.height;

    /** If custom width or height for the activity indicator is not coming from styles,default size (i.e. 40) is used. */
    if (!activityIndicatorSize) {
        activityIndicatorSize = ACTIVITY_INDICATOR_SIZE;
    }

    return (
        <div
            style={ styles.container }
            className="xha-activity-indicator"
            data-testid={ testID && `${testID}__container` }>
            <ThemeProvider theme={ theme }>
                <CircularProgress
                    data-testid={ testID && `${testID}__activityIndicator` }
                    size={ activityIndicatorSize }
                    color={ ACTIVITY_INDICATOR_COLOR }
                    thickness={ thickness } />
            </ThemeProvider>
        </div>
    );
};

ActivityIndicator.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the Activity Indicator container */
        container: PropTypes.object,

        /** Styles for the Activity Indicator */
        activityIndicator: PropTypes.object,
    }),

    /** Thickness Of the Activity Indicator. */
    thickness: PropTypes.number,

    /** Id used for testing */
    testID: PropTypes.string,
};

ActivityIndicator.defaultProps = {
    contextStyles: {},
    thickness: ACTIVITY_INDICATOR_THICKNESS,
};

export default ActivityIndicator;
