import React from 'react';
import PropTypes from 'prop-types';
import ButtonBase from '../ButtonBase/ButtonBase';
import Icon from '../Icon/Icon';
import TextLabel from '../TextLabel/TextLabel';
import getStyles from './Button.styles';


/**
 * Button component: A control is that shows a user a button with a text or icon representation of an action to be performed and allows the user to select the button to run the action.
 * The control is capable of displaying an activity indicator to indicate that the performed action is in progress.
 * @see https://material-ui.com/demos/buttons/
 * @see https://material-ui.com/api/button/
 * @see https://material-ui.com/api/button-base/
 */
const Button = (props) => {
    const {
        contextStyles,
        disabled,
        iconName,
        iconSize,
        onClick,
        reverse,
        testID,
        text,
        variant,
    } = props;

    const styles = getStyles(contextStyles);

    /** Renders the Text */
    const renderText = () => {
        const textProps = {
            contextStyles: {
                container: styles.textContainer,
                text: styles.text,
            },
        };
        if (testID) { textProps['data-testid'] = `${testID}__textLabel`; }

        return (
            <TextLabel { ...textProps }>
                { text }
            </TextLabel>
        );
    };

    /** Renders the Icon */
    const renderIcon = () => {
        const iconProps = {
            contextStyles: {
                container: styles.iconContainer,
                icon: styles.icon,
            },
            iconName,
            iconSize,
            title: iconName,
        };
        if (testID) { iconProps['data-testid'] = `${testID}__icon`; }

        return (
            <Icon { ...iconProps } />
        );
    };

    // Generate button props
    const buttonProps = {
        contextStyles: {
            container: styles.container,
            contentContainer: styles.contentContainer,
        },

        // Generate additional MuiButton theme information for the ButtonBase component
        theme: {
            root: {
                ...styles.text,
            },
        },
        variant,
    };
    if (disabled) { buttonProps.disabled = true; }
    if (!disabled) { buttonProps.onClick = onClick; }
    if (testID) { buttonProps.testID = testID; }

    /** If text or icon are not provided, the container is also not rendered. */
    if (text || iconName) {
        return (
            <ButtonBase { ...buttonProps }>
                { (!!iconName && !reverse) && renderIcon() }
                { !!text && renderText() }
                { (!!iconName && reverse) && renderIcon() }
            </ButtonBase>
        );
    }

    return null;
};

Button.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main button container */
        container: PropTypes.object,

        /** Styles for the container around the button text and icon */
        contentContainer: PropTypes.object,

        /** Styles for icon */
        icon: PropTypes.object,

        /** Styles for the container around the icon */
        iconContainer: PropTypes.object,

        /** Styles for the button text */
        text: PropTypes.object,

        /** Styles for the container around the button text */
        textContainer: PropTypes.object,
    }),

    /** Disable the button */
    disabled: PropTypes.bool,

    /** The icon to be displayed */
    iconName: PropTypes.string,

    /**
     * Size of the icon to be rendered.
     */
    iconSize: PropTypes.oneOf([
        'inherit',
        'default',
        'small',
        'large',
    ]),

    /**
     * Called when the button is clicked.
     * @param {Object} props - The button instance props
     * @param {SyntheticEvent} event - The click event
     */
    onClick: PropTypes.func.isRequired,

    /** Icon location. Defaults to the left (false). */
    reverse: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Text displayed inside the button */
    text: PropTypes.string,

    /** Button Variant. Defaults to the 'contained' variant. Valid Types: 'outlined', 'contained' */
    variant: PropTypes.oneOf([
        'contained',
        'outlined',
    ]),
};

Button.defaultProps = {
    contextStyles: {},
    iconSize: 'small',
    variant: 'contained',
};

export default Button;
