import React from 'react';
import PropTypes from 'prop-types';
import MuiButton from '@material-ui/core/Button';
import { MuiThemeProvider, createMuiTheme } from '@material-ui/core/styles';
import getStyles from './ButtonBase.styles';


/**
 * Button component: A control is that shows a user a button with a text or icon representation of an action to be performed and allows the user to select the button to run the action.
 * The control is capable of displaying an activity indicator to indicate that the performed action is in progress.
 * @see https://material-ui.com/demos/buttons/
 * @see https://material-ui.com/api/button/
 * @see https://material-ui.com/api/button-base/
 */
const ButtonBase = (props) => {
    const {
        children,
        contextStyles,
        disabled,
        onClick,
        testID,
        theme: xTheme,
        variant,
    } = props;

    const styles = getStyles(contextStyles);

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiButton: {
                // Name of the rule
                outlined: {
                    ...styles.contentContainer,
                },
                contained: {
                    ...styles.contentContainer,
                },
                ...xTheme,
            },
        },
    });

    // Generate container props
    const containerProps = {
        style: styles.container,
    };
    if (testID) { containerProps['data-testid'] = `${testID}__container`; }

    // Generate button props
    const buttonProps = {
        variant,
    };
    if (disabled) { buttonProps.disabled = true; }
    if (!disabled) { buttonProps.onClick = onClick; }
    if (testID) { buttonProps['data-testid'] = `${testID}__button`; }


    return (
        <div { ...containerProps }>

            <MuiThemeProvider theme={ theme }>
                <MuiButton { ...buttonProps }>
                    { children }
                </MuiButton>
            </MuiThemeProvider>

        </div>
    );
};

ButtonBase.propTypes = {
    /** Contents For Button */
    children: PropTypes.oneOfType([
        PropTypes.node,
        PropTypes.arrayOf(PropTypes.node),
    ]).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main button container */
        container: PropTypes.object,

        /** Styles for the container around the button content */
        contentContainer: PropTypes.object,
    }),

    /** Disable the button */
    disabled: PropTypes.bool,

    /**
     * Called when the button is clicked.
     * @param {Object} props - The button instance props
     * @param {SyntheticEvent} event - The click event
     */
    onClick: PropTypes.func.isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** MuiButton theme information */
    theme: PropTypes.object,

    /** Button Variant. Defaults to the 'contained' variant. Valid Types: 'outlined', 'contained' */
    variant: PropTypes.oneOf([
        'contained',
        'outlined',
    ]),
};

ButtonBase.defaultProps = {
    contextStyles: {},
    variant: 'contained',
};

export default ButtonBase;
