import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { ThemeProvider } from '@material-ui/styles';
import { createMuiTheme, IconButton, InputAdornment, MenuItem, ClickAwayListener, Grow, Paper, Popper, MenuList } from '@material-ui/core';
import ArrowDropDown from '@material-ui/icons/ArrowDropDown';

// Import TextField component
import TextField from '../TextField/TextField';

// Styles
import getStyles from './ComboBox.styles';

/**
 * A ComboBox is a component that allows a user to select an option from a pre-defined menu of options as well as enter their own custom value.
 * @see https://material-ui.com/demos/text-fields/#customized-inputs for the Material-UI Customized Text Inputs.
 */
class ComboBox extends Component {
    state = {
        isMenuOpen: false,
        anchorEl: null,
    };

    render() {
        const {
            contextStyles,
            items,
            testID,
            disabled,
            renderItem,
            onChange,
            value,
            autoFocus,
        } = this.props;

        const styles = getStyles(contextStyles);
        const theme = this.getTheme(styles);
        const {
            isMenuOpen,
            anchorEl,
        } = this.state;

        const {
            container,
            textFieldContainer,
            popper,
        } = styles;

        return (
            <div style={ container }>
                <TextField
                    disabled={ disabled }
                    onChange={ onChange }
                    value={ value }
                    endAdornment={ (
                        <InputAdornment position="end">
                            <IconButton
                                onClick={ this.handleClick }
                                disabled={ disabled }>
                                <ArrowDropDown />
                            </IconButton>
                        </InputAdornment>
                    ) }
                    autoFocus={ autoFocus }
                    testID={ testID && `${testID}__textField` }
                    contextStyles={ {
                        container: textFieldContainer,
                    } } />
                <ThemeProvider theme={ theme }>
                    <Popper
                        data-testid={ testID && `${testID}__popper` }
                        anchorEl={ anchorEl }
                        open={ isMenuOpen }
                        transition
                        disablePortal
                        style={ popper }>
                        { ({ TransitionProps }) => (
                            <Grow { ...TransitionProps }>
                                <Paper>
                                    <ClickAwayListener onClickAway={ this.handleClose }>
                                        <MenuList data-testid={ testID && `${testID}__menuList` }>
                                            { items.length === 0 && (
                                                <MenuItem
                                                    value=""
                                                    onClick={ this.handleClose }>
                                                    (empty)
                                                </MenuItem>
                                            ) }
                                            { items.map((item) => renderItem(MenuItem, item, this.handleClose)) }
                                        </MenuList>
                                    </ClickAwayListener>
                                </Paper>
                            </Grow>
                        ) }
                    </Popper>
                </ThemeProvider>
            </div>
        );
    }

    /** Helper method to close the menu */
    handleClose = () => {
        this.setState({
            isMenuOpen: false,
            anchorEl: null,
        });
    };

    /** Called when the ArrayDropDown Icon is clicked to open the menu */
    handleClick = (event) => {
        // Set the anchor element to the TextField component below which we need to show the Popper component
        const anchorTarget = event.currentTarget.parentNode.parentNode.parentNode;
        this.setState({
            isMenuOpen: true,
            anchorEl: anchorTarget,
        });
    };

    /**
     * Helper method to get the theme
     * @param {Object} styles - Styles that are used to create the theme
     */
    getTheme = (styles) => createMuiTheme({
        overrides: {
            MuiMenuItem: {
                root: {
                    ...styles.menuOptions,
                    '&$selected': {
                        ...styles.selectedItem,
                    },
                },
            },
            MuiPaper: {
                root: {
                    ...styles.menu,
                },
            },
        },
        typography: {
            useNextVariants: true,
        },
    });
}

ComboBox.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the ComboBox container */
        container: PropTypes.object,

        /** Styles for the menu container */
        menu: PropTypes.object,

        /** Styles for the menu Options */
        menuOptions: PropTypes.object,

        /** Styles for the Selected drop down option text */
        selectedItem: PropTypes.object,
    }),

    /** Prevents the ComboBox from accepting input */
    disabled: PropTypes.bool,

    /**
     * Called when the ComboBox TextField's text changes
     * @param {Event} event - TextField input change event
     */
    onChange: PropTypes.func.isRequired,

    /** The text to populate in the TextField */
    value: PropTypes.string,

    /** Array of items for the dropdown menu */
    items: PropTypes.array,

    /**
     * Called to render the menu items
     * @param {Component} MenuItem - The default menu item component
     * @param {any} Item - Item that needs to be rendered
     * @param {Function} onClick - Function that needs to be called on click of Menu Item in order to close the menu
     */
    renderItem: PropTypes.func.isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Auto Focuses on the input */
    autoFocus: PropTypes.bool,
};

ComboBox.defaultProps = {
    contextStyles: {},
    items: [],
    value: '',
};

export default ComboBox;
