import React, { Component } from 'react';
import PropTypes from 'prop-types';
import 'date-fns';
import DateFnsUtils from '@date-io/date-fns';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import { MuiPickersUtilsProvider, KeyboardTimePicker, KeyboardDatePicker, KeyboardDateTimePicker } from '@material-ui/pickers';

// Styles
import getStyles from './DateTimePicker.styles';

/**
 * DateTimePicker component for picking date and time from user
 * @see https://material-ui.com/demos/pickers/#material-ui-pickers
 * @see https://material-ui-pickers.dev/getting-started/usage
 */
class DateTimePicker extends Component {
    constructor(props) {
        super(props);
        const {
            selectedDate,
            autoFocus,
        } = props;
        this.state = {
            selectedDate: selectedDate,
            isDateSelected: !!selectedDate,
            isPickerClosed: false,
            open: autoFocus,
        };
    }

    render() {
        const {
            contextStyles,
            format,
            is24Hour,
            mode,
            testID,
        } = this.props;

        const {
            selectedDate,
            open,
        } = this.state;

        const styles = getStyles(contextStyles);

        const theme = createMuiTheme({
            overrides: {
                MuiFormControl: {
                    root: {
                        ...styles.container,
                    },
                },
                MuiInputBase: {
                    root: {
                        ...styles.inputContainer,
                    },
                    input: {
                        ...styles.inputText,
                    },
                },
                MuiIconButton: {
                    root: {
                        ...styles.iconContainer,
                    },
                    label: {
                        ...styles.icon,
                    },
                },
                MuiPickersToolbar: {
                    toolbar: {
                        ...styles.pickerToolbar,
                    },
                },
                MuiPickerDTTabs: {
                    tabs: {
                        ...styles.pickerTabs,
                    },
                },
                MuiPickersModal: {
                    dialogRoot: {
                        ...styles.pickerModalDialog,
                    },
                },
                MuiDialogActions: {
                    root: {
                        ...styles.pickerModalDialogAction,
                    },
                },
                MuiPickersCalendarHeader: {
                    switchHeader: {
                        ...styles.calenderHeader,
                    },
                },
                MuiPickersDay: {
                    day: {
                        ...styles.calenderDay,
                    },
                    daySelected: {
                        ...styles.calenderDayIsSelected,
                    },
                },
                MuiPickersClock: {
                    container: {
                        ...styles.clockContainer,
                    },
                    clock: {
                        ...styles.clock,
                    },
                    pin: {
                        ...styles.clockPin,
                    },
                },
                MuiPickersClockNumber: {
                    clockNumber: {
                        ...styles.clockNumber,
                    },
                    '&$selected': {
                        ...styles.clockNumberSelected,
                    },
                },
                MuiPickersClockPointer: {
                    pointer: {
                        ...styles.clockPointer,
                    },
                    thumb: {
                        ...styles.clockThumb,
                    },
                    noPoint: {
                        ...styles.clockNoPoint,
                    },
                },
            },
            typography: {
                useNextVariants: true,
            },
        });

        const dateProps = {
            'data-testid': testID,
            value: selectedDate,
            ampm: !is24Hour,
            okLabel: 'Set',
            format,
            onChange: this.handleChange,
            onAccept: this.handleSelect,
            onClose: this.handleClose,
            onOpen: this.handleOpen,
            open,
        };

        return (
            <MuiPickersUtilsProvider utils={ DateFnsUtils }>
                <ThemeProvider theme={ theme }>
                    { mode === 'date' &&
                    <KeyboardDatePicker { ...dateProps } />
                    }
                    { mode === 'time' &&
                    <KeyboardTimePicker { ...dateProps } />
                    }
                    { mode === 'datetime' &&
                    <KeyboardDateTimePicker { ...dateProps } />
                    }
                </ThemeProvider>
            </MuiPickersUtilsProvider>
        );
    }

    componentDidUpdate(prevProps, prevState) {
        const {
            selectedDate,
            isPickerClosed,
            isDateSelected,
        } = this.state;

        // Update the selectedDate state when 'selectedDate' prop is different
        if (selectedDate !== prevState.selectedDate) {
            // when date is not selected and picker is closed
            if (isPickerClosed && !isDateSelected) {
                this.updateDate(null);
            }
            else {
                this.updateDate(selectedDate);
            }
        }
    }

    /**
    * Helper method to update date
    * @param {Object} selectedDate - The new value
    */
    updateDate = (selectedDate) => {
        this.setState({ selectedDate });
    };

    /**
    * Called when the input changes
    * @param {Object} selectedDate - The new value
    */
    handleChange = (selectedDate) => {
        const { onChange } = this.props;

        this.setState({ selectedDate }, () => {
            if (onChange) {
                onChange(selectedDate);
            }
        });
    };

    // Called when the picker is closed
    handleClose = () => {
        this.setState({
            isPickerClosed: true,
            open: false,
        });
    };

    // Called when the picker is opened for selection changes
    handleOpen = () => {
        const { selectedDate } = this.state;

        this.setState({
            isPickerClosed: false,
            isDateSelected: !!selectedDate,
            open: true,
        });
    };

    // Called when the date is selected
    handleSelect = () => {
        this.setState({
            isDateSelected: true,
            isPickerClosed: true,
        });
    };
}

DateTimePicker.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the label and picker */
        container: PropTypes.object,

        /** Styles for the container surrounding the input */
        inputContainer: PropTypes.object,

        /** Styles for the input text */
        inputText: PropTypes.object,

        /** Styles for the container surrounding the icon */
        iconContainer: PropTypes.object,

        /** Styles for the icon */
        icon: PropTypes.object,

        /** Styles for the picker toolbar */
        pickerToolbar: PropTypes.object,

        /** Styles for the picker tabs */
        pickerTabs: PropTypes.object,

        /** Styles for the picker modal dialog */
        pickerModalDialog: PropTypes.object,

        /** Styles for the picker modal action footer*/
        pickerModalDialogAction: PropTypes.object,

        /** Styles for the calender header */
        calenderHeader: PropTypes.object,

        /** Styles for the calender day */
        calenderDay: PropTypes.object,

        /** Styles for the selected calender day */
        calenderDayIsSelected: PropTypes.object,

        /** Styles for the clock container */
        clockContainer: PropTypes.object,

        /** Styles for the clock */
        clock: PropTypes.object,

        /** Styles for the clock pin */
        clockPin: PropTypes.object,

        /** Styles for the clock pointer */
        clockPointer: PropTypes.object,

        /** Styles for the clock thumb */
        clockThumb: PropTypes.object,

        /** Styles for the clock pointer end*/
        clockNoPoint: PropTypes.object,

        /** Styles for the clock number */
        clockNumber: PropTypes.object,

        /** Styles for the clock number selected */
        clockNumberSelected: PropTypes.object,
    }),

    /** Custom display format (see NPM package date-fns) */
    format: PropTypes.string,

    /** Display the date picker in a 24 hour format */
    is24Hour: PropTypes.bool,

    /** Date type format: date, time, datetime */
    mode: PropTypes.oneOf([
        'date',
        'time',
        'datetime',
    ]),

    /** Selected date value */
    selectedDate: PropTypes.instanceOf(Date),

    /**
    * Called when the picker date has changed
    * @param {Date} date - The selected date
    */
    onChange: PropTypes.func,

    /** Auto Focuses on the input */
    autoFocus: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,
};

DateTimePicker.defaultProps = {
    contextStyles: {},
    format: 'M/dd/yy, h:mm a',
    mode: 'date',
    autoFocus: false,
    selectedDate: null,
};

export default DateTimePicker;
