import React from 'react';
import PropTypes from 'prop-types';
import TextField from '@material-ui/core/TextField';
import InputAdornment from '@material-ui/core/InputAdornment';
import IconButton from '@material-ui/core/IconButton';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import CloudUploadIcon from '@material-ui/icons/CloudUpload';

// Styles
import getStyles from './FilePicker.styles';

/**
 * FilePicker component allows user to pick files to upload.
 * @see https://material-ui.com/demos/buttons/
 */
class FilePicker extends React.Component {
    state = {
        fileName: '',
    };

    render() {
        const { fileName } = this.state;
        const {
            fileTypes,
            contextStyles,
            testID,
        } = this.props;

        let accept = '';

        const fileMap = {
            pdf: 'application/pdf,',
            plainText: 'text/plain,',
            images: 'image/*,',
            audio: 'audio/*,',
            video: 'video/*,',
        };

        fileTypes.forEach( (fileType) => accept = accept + ( fileMap[fileType] || '*,' ));

        const styles = getStyles(contextStyles);

        const theme = createMuiTheme({
            typography: {
                // https://material-ui.com/style/typography/#migration-to-typography-v2
                useNextVariants: true,
            },
            overrides: {
                MuiFormControl: {
                    root: {
                        ...styles.container,
                    },
                },
                MuiFormLabel: {
                    root: {
                        ...styles.labelContainer,
                        ...styles.labelText,
                    },
                },
                MuiInputBase: {
                    root: {
                        ...styles.inputContainer,
                    },
                    input: {
                        ...styles.inputText,
                    },
                },
                MuiIconButton: {
                    root: {
                        ...styles.iconContainer,
                    },
                    label: {
                        ...styles.icon,
                    },
                },
            },
        });

        return (
            <ThemeProvider theme={ theme }>
                <input
                    accept={ accept }
                    ref={ (input) => this.inputElement = input }
                    style={ { display: 'none' } }
                    type="file"
                    onChange={ this.handleChange } />

                <TextField
                    value={ fileName }
                    readOnly
                    type="text"
                    data-testid={ testID }
                    placeholder="Choose a file"
                    InputProps={ {
                        readOnly: true,
                        endAdornment: (
                            <InputAdornment position="end">
                                <IconButton onClick={ this.handleClick }>
                                    <CloudUploadIcon />
                                </IconButton>
                            </InputAdornment>
                        ),
                    } } />
            </ThemeProvider>
        );
    }

    /**
     * Called when the file input changes
     * @param {{target: Object}} e - The new file
     */
    handleChange = (e) => {
        const { onChange } = this.props;
        const { value } = e.target;
        if (value) {
            const file = value.replace(/^.*[\\/]/, '');
            this.setState({ fileName: file });
        }
        else {
            this.setState({ fileName: '' });
        }
        if (onChange) {
            onChange(e);
        }
    };

     /** Called when the picker Icon is clicked to open the picker */
     handleClick = () => {
         this.inputElement.click();
     }
}

FilePicker.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the picker */
        container: PropTypes.object,

        /** Styles for the container surrounding the input */
        inputContainer: PropTypes.object,

        /** Styles for the input text */
        inputText: PropTypes.object,

        /** Styles for the container surrounding the icon */
        iconContainer: PropTypes.object,

        /** Styles for the icon */
        icon: PropTypes.object,
    }),

    /**
     * Called when the document selection has changed
     * @param {SyntheticEvent} event - The browser document change event
     */
    onChange: PropTypes.func.isRequired,

    /** File input types*/
    fileTypes: PropTypes.arrayOf(PropTypes.oneOf([
        'allFiles',
        'images',
        'plainText',
        'audio',
        'video',
        'pdf',
    ])),

    /** Id used for testing */
    testID: PropTypes.string,
};

FilePicker.defaultProps = {
    fileTypes: [ 'allFiles' ],
};

export default FilePicker;
