import React from 'react';
import { MuiThemeProvider, createMuiTheme } from '@material-ui/core';
import MuiIconButton from '@material-ui/core/IconButton';
import PropTypes from 'prop-types';
import Icon from '@material-ui/core/Icon';

// Styles
import getStyles from './IconButton.styles';
import './style.css';

/** Default size of Icon button */
const ICONBUTTON_ICON_SIZE_DEFAULT = 'default';

/** Default ripple property. It is enabled by default. */
const ICONBUTTON_DISABLE_RIPPLE = false; // Ripple is enabled out of the box. If ripple is not required, it can be set via props.

/** Default state of Icon button. The icon button is enabled out of the box. */
const ICONBUTTON_IS_DISABLED = false;

/**
 * Icon Button component :allows a user to view a non-image icon and have a click action associated with it.
 * @see https://material-ui.com/demos/buttons/#icon-buttons
 * @see https://material-ui.com/style/icons/
 */
const IconButton = (props) => {
    const {
        contextStyles,
        iconName,
        title,
        iconSize,
        disabled,
        disableRipple,
        onClick,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiIconButton: { // Name of the component ⚛️ / style sheet
                root: {// Name of the rule
                    ...styles.button,
                    ...styles.icon,
                },
            },
        },
    });

    const iconTitle = title || iconName;

    /** If iconName prop is undefined, the container is also not rendered. */
    if (iconName) {
        return (
            <div
                style={ styles.container }
                className="icon-button"
                data-testid={ testID && `${testID}__container` }>
                <MuiThemeProvider theme={ theme }>
                    <MuiIconButton
                        disableRipple={ disableRipple }
                        disabled={ disabled }
                        data-testid={ testID && `${testID}__iconButton` }
                        onClick={ onClick }>
                        <Icon
                            fontSize={ iconSize }
                            data-testid={ testID && `${testID}__icon` }
                            title={ iconTitle }>
                            { iconName }
                        </Icon>
                    </MuiIconButton>
                </MuiThemeProvider>
            </div>
        );
    }

    return null;
};

IconButton.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the icon */
        container: PropTypes.object,

        /** Styles for a icon */
        icon: PropTypes.object,
    }),

    /**
     * Name of a XHA UI icon, or an icon element, to render.
     * This prop is required because it determines the content of the component, but it can
     * be explicitly set to falsy values to render nothing.
     * If `null` or `undefined` or `false`, this component will render nothing.
     */
    iconName: PropTypes.string.isRequired,

    /**
     * Description string.
     * Browsers usually render this as a tooltip on hover, whereas screen
     * readers will use it for aural feedback.
     * By default, this is set to the icon's name for accessibility.
     */
    title: PropTypes.string,

    /** Size of the icon to be rendered. */
    iconSize: PropTypes.oneOf([
        'inherit',
        'default',
        'small',
        'large',
    ]),

    /** For disabling the ripple effect when the icon is clicked. */
    disableRipple: PropTypes.bool,

    /** For disabling the icon button. */
    disabled: PropTypes.bool,

    /**
     * Called when the icon is clicked
     * @param {Object} props - The button instance props
     * @param {SyntheticEvent} event - The click event
     */
    onClick: PropTypes.func,

    /** Id used for testing. */
    testID: PropTypes.string,
};

IconButton.defaultProps = {
    contextStyles: {},
    disabled: ICONBUTTON_IS_DISABLED,
    disableRipple: ICONBUTTON_DISABLE_RIPPLE,
    iconSize: ICONBUTTON_ICON_SIZE_DEFAULT,
};

export default IconButton;