/* eslint-disable jsx-a11y/click-events-have-key-events */
import React, { Component } from 'react';
import PropTypes from 'prop-types';

// Styles
import getStyles from './Image.styles';
import ActivityIndicator from '../ActivityIndicator/ActivityIndicator';

/**
 * Image Component Class, that will render the supplied image url. While Image is loading it will show
 * a loader meanwhile
 * @returns {React.ComponentClass}
 */
class Image extends Component {
    static defaultProps = {
        contextStyles: {},
        showLoadingIndicator: true,
    };

    state = {
        /**  local state to handle if the Image is Loaded or not */
        isImageLoaded: false,
    };

    render() {
        const {
            showLoadingIndicator,
            isDisabled,
            contextStyles,
        } = this.props;
        const styles = getStyles(contextStyles, isDisabled);
        const { isImageLoaded } = this.state;
        return (

            // container of the image
            // eslint-disable-next-line jsx-a11y/no-static-element-interactions
            <div
                style={ styles.container }
                onClick={ this.handleClick }>
                { /** When showIndicator is true and Image is not loaded Should show the Loading indicator */ }
                { showLoadingIndicator && !isImageLoaded
                    ? this.renderLoadingIndicator(styles.loadingIndicator)
                    : null }
                { this.renderImage(styles.image) }
            </div>
        );
    }

    /** Render the Loading indicator*/
    renderLoadingIndicator = (styles) => <ActivityIndicator contextStyles={ { activityIndicator: styles } } />;

    /**
     * Render the Image inside the container div
     */
    renderImage = (styles) => {
        const {
            imageSrc, testID,
        } = this.props;
        return (
            <img
                src={ imageSrc }
                alt={ imageSrc }
                data-testid={ testID }
                style={ styles }
                onLoad={ this.handleImageLoaded } />
        );
    };

    /**
     * helper method to acknowledge that image has been loaded and
     * Change the state to hide the Loading indicator
     */
    handleImageLoaded = () => {
        this.setState({ isImageLoaded: true });
    };

    /**
     * Called when the image place holder is Clicked
     * @param {{ target: Object }} event - The browser click event
     */
    handleClick = (event) => {
        const {
            onClick, isDisabled,
        } = this.props;
        if (onClick && !isDisabled) {
            onClick(event);
        }
    };
}

Image.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the image */
        container: PropTypes.object,

        /** Styles for the loading indicator */
        loadingIndicator: PropTypes.object,

        /** Styles for the image */
        image: PropTypes.object,
    }),

    /** The image source (either a remote URL or a local file resource) */
    imageSrc: PropTypes.string.isRequired,

    /** Disables image touch */
    isDisabled: PropTypes.bool,

    /** Controls the loading indicator visibility */
    showLoadingIndicator: PropTypes.bool,

    /**
    * Called when the image was pressed
    * @param {Function} props - The component props
    * @param {onclick} event - The native press event
    */
    onClick: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

export default Image;
