import React from 'react';
import PropTypes from 'prop-types';
import Modal from '../Modal/Modal';
import TextLabel from '../TextLabel/TextLabel';
import ActivityIndicator from '../ActivityIndicator/ActivityIndicator';
import getStyles from './PageActivityIndicator.styles';

/**
 * A control that indicates to the user that a page level action is currently being performed with a loading text and an activity indicator.
 * @see https://material-ui.com/demos/dialogs/
 * @see https://material-ui.com/components/progress/
 */
const PageActivityIndicator = (props) => {
    const {
        contextStyles,
        isOpen,
        isTransparentOverlay,
        testID,
        activityIndicatorProps,
        modalProps,
        textLabelProps,
    } = props;
    const styles = getStyles(contextStyles);

    const loadingTextString = 'One moment as the page is being constructed...';

    /*
        Unless you want to override core-styles of the sub components, you should use contextStyles from props to pass styles to sub-components.
        If you want to override core-styles of the sub components, you should use getStyles which returns core-styles and contextStyles from props combined, to pass styles to sub-components.
        In this case core-styles for container of activityIndicator and text of textLabel are overridden.
     */
    const activityIndicatorStyles = {
        activityIndicator: contextStyles.spinningIndicator,
        container: styles.spinningIndicatorContainer,
    };
    const modalStyles = {
        modal: contextStyles.modal,
        overlay: contextStyles.overlay,
    };
    const textLabelStyles = {
        text: styles.loadingText,
        container: contextStyles.loadingTextContainer,
    };

    return (
        <Modal
            { ...modalProps }
            data-testid={ testID && `${testID}__modal` }
            isClosable={ false }
            contextStyles={ modalStyles }
            isTransparentOverlay={ isTransparentOverlay }
            isModalOpen={ isOpen }>
            <TextLabel
                { ...textLabelProps }
                data-testid={ testID && `${testID}__loadingText` }
                contextStyles={ textLabelStyles }>
                { loadingTextString }
            </TextLabel>
            <ActivityIndicator
                { ...activityIndicatorProps }
                data-testid={ testID && `${testID}__activityIndicator` }
                contextStyles={ activityIndicatorStyles } />
        </Modal>
    );
};

PageActivityIndicator.defaultProps = {
    contextStyles: {},
};

PageActivityIndicator.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the overlay */
        overlay: PropTypes.object,

        /** Styles for the modal container */
        modal: PropTypes.object,

        /** Styles for loading text */
        loadingText: PropTypes.object,

        /** Styles for spinning indicator */
        spinningIndicator: PropTypes.object,

        /** Styles for loading text */
        loadingTextContainer: PropTypes.object,

        /** Styles for spinning indicator */
        spinningIndicatorContainer: PropTypes.object,
    }),

    /** Defines open or closed state of page activity indicator */
    isOpen: PropTypes.bool.isRequired,

    /** Changes the overlay darkened fill to transparent */
    isTransparentOverlay: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Props for ActivityIndicator */
    activityIndicatorProps: PropTypes.shape({
        /** Thickness of spinner for activity indicator */
        thickness: PropTypes.number,
    }),

    /** Props for Modal */
    modalProps: PropTypes.shape({
        /** If modal is closable by user on click of overlay or ESC keyPress. */
        isClosable: PropTypes.bool,

        /** Called when modal is closable and user closes the modal. */
        onClose: PropTypes.func,
    }),

    /** Props for TextLabel */
    textLabelProps: PropTypes.shape({
        /** The text to display */
        children: PropTypes.string,

        /**
         * Used to truncate the text with an ellipsis after computing the text layout, including line wrapping,
         * such that the total number of lines does not exceed this number
         */
        numberOfLines: PropTypes.number,
    }),
};

export default PageActivityIndicator;
