import React from 'react';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import MuiTextField from '@material-ui/core/TextField';
import PropTypes from 'prop-types';

// Styles
import getStyles from './TextField.styles';

/**
 * TextField is a component that allows users to add or modify the input.
 * @see https://material-ui.com/demos/text-fields/
 */
const TextField = (props) => {
    const {
        contextStyles,
        autoGrow,
        maxLength,
        autoFocus,
        disabled,
        type,
        multiline,
        rowsMax,
        testID,
        endAdornment,
        startAdornment,
        value,
        onChange,
    } = props;

    const styles = getStyles(contextStyles, autoGrow, value);
    const theme = createMuiTheme({
        overrides: {
            MuiFormControl: {
                root: {
                    ...styles.container,
                },
            },
            MuiInput: {
                root: {
                    ...styles.input,
                },
            },
        },
        typography: {
            useNextVariants: true,
        },
    });

    return (
        <ThemeProvider theme={ theme }>
            <MuiTextField
                data-testid={ testID }
                autoFocus={ autoFocus }
                disabled={ disabled }
                type={ type }
                multiline={ multiline }
                rowsMax={ rowsMax }
                onChange={ onChange }
                value={ value }
                InputProps={ {
                    endAdornment,
                    startAdornment,
                    inputProps: { maxLength },
                } } />
        </ThemeProvider>
    );
};

TextField.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the text field */
        container: PropTypes.object,

        /** Styles for the text field */
        input: PropTypes.object,
    }),

    /** Focuses the input */
    autoFocus: PropTypes.bool,

    /** Grows/shrinks the input based on the size of its text value */
    autoGrow: PropTypes.bool,

    /** Prevents the text field from accepting input */
    disabled: PropTypes.bool,

    /** Determines the type of input */
    type: PropTypes.oneOf([
        'text',
        'password',
        'email',
        'number',
    ]),

    /** Limits the maximum number of characters that can be entered */
    maxLength: PropTypes.number,

    /** Enables the text input to be multiple lines */
    multiline: PropTypes.bool,

    /** Maximum number of rows in case of multiline */
    rowsMax: PropTypes.number,

    /**
     * Called when the TextField's input changes
     * @param {Event} event - TextField's onChange event
     */
    onChange: PropTypes.func.isRequired,

    /** The text to populate in the input */
    value: PropTypes.string.isRequired,

    /** Elements that needs to be placed towards the end of the TextField */
    endAdornment: PropTypes.node,

    /** Elements that needs to be placed at the beginning of the TextField */
    startAdornment: PropTypes.node,

    /** Id used for testing */
    testID: PropTypes.string,
};

TextField.defaultProps = {
    contextStyles: {},
    type: 'text',
};

export default TextField;
