import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { createMuiTheme, Switch } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';

// Styles
import getStyles from './Toggle.styles';

/**
 * A control that allows a user to toggle between binary values (on/off, yes/no, true/false)
 * @see https://material-ui.com/components/switches/
 */
class Toggle extends Component {
    render() {
        const {
            contextStyles,
            disabled,
            testID,
            defaultChecked,
            checked,
        } = this.props;

        const styles = getStyles(contextStyles);
        const theme = this.getTheme(styles);

        return (
            <div style={ { ...styles.container } }>
                <ThemeProvider theme={ theme }>
                    <Switch
                        data-testid={ testID }
                        onChange={ this.handleChange }
                        disabled={ disabled }
                        defaultChecked={ defaultChecked }
                        checked={ checked } />
                </ThemeProvider>
            </div>
        );
    }

    /**
     * Helper method to get theme
     * @param {Object} styles object returned by getStyles
     */
    getTheme = (styles) => createMuiTheme({
        typography: {
            useNextVariants: true,
        },
        overrides: {
            MuiSwitch: {
                root: {
                    // style override for toggleContainer.
                    ...styles.input,
                },
                colorSecondary: {
                    // style override for toggle.
                    '&$checked': {
                        // change toggle thumbColor
                        color: styles.input.thumbColor,
                        '& + $bar': {
                            // change toggle trackColor
                            backgroundColor: styles.input.trackColor,
                        },
                    },
                },
            },
        },
    });

    /**
     * Called when state of toggle component is changed.
     * @param {SyntheticEvent} e - The native change event
     */
    handleChange = (e) => {
        const {
            onChange,
            disabled,
        } = this.props;

        const { checked } = e.target;

        if (!disabled) {
            if (onChange) {
                onChange(checked);
            }
        }
    }
}

Toggle.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the label and input toggle */
        container: PropTypes.object,

        /** Styles for a switch component */
        input: PropTypes.object,
    }),

    /** The default switch value */
    defaultChecked: PropTypes.bool,

    /** The switch value */
    checked: PropTypes.bool,

    /** Disables the control */
    disabled: PropTypes.bool,

    /**
     * Called when the switch was changed
     * @param {Boolean} value - The current switch value: on = true, off = false
     */
    onChange: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

Toggle.defaultProps = {
    contextStyles: {},
};

export default Toggle;
